#include "SonarExpert.h"
#include "math.h"

#define C1	57
#define C2	5

int initport(int fd) {
	struct termios options;
	/*// Get the current options for the port...
	tcgetattr(fd, &options);
	// Set the baud rates to 19200...
	cfsetispeed(&options, B19200);
	// Enable the receiver and set local mode...
	options.c_cflag |= (CLOCAL | CREAD);

	options.c_cflag &= ~PARENB;
	options.c_cflag &= ~CSTOPB;
	options.c_cflag &= ~CSIZE;
	options.c_cflag |= CS8;

	// Set the new options for the port...
	tcsetattr(fd, TCSANOW, &options);*/
	bzero(&options, sizeof(termios));
        cfmakeraw(&options);
        options.c_cflag = B19200 | CS8 | CLOCAL | CREAD;
        options.c_iflag = IGNPAR | ICRNL;
        options.c_oflag = 0;
        options.c_lflag &= ~ICANON;
	return 1;
}

int getbaud(int fd) {
	struct termios termAttr;
	int inputSpeed = -1;
	speed_t baudRate;
	tcgetattr(fd, &termAttr);
	/* Get the input speed.                              */
	baudRate = cfgetispeed(&termAttr);
	switch (baudRate) {
		case B0:      inputSpeed = 0; break;
		case B50:     inputSpeed = 50; break;
		case B110:    inputSpeed = 110; break;
		case B134:    inputSpeed = 134; break;
		case B150:    inputSpeed = 150; break;
		case B200:    inputSpeed = 200; break;
		case B300:    inputSpeed = 300; break;
		case B600:    inputSpeed = 600; break;
		case B1200:   inputSpeed = 1200; break;
		case B1800:   inputSpeed = 1800; break;
		case B2400:   inputSpeed = 2400; break;
		case B4800:   inputSpeed = 4800; break;
		case B9600:   inputSpeed = 9600; break;
		case B19200:  inputSpeed = 19200; break;
		case B38400:  inputSpeed = 38400; break;
	}
	return inputSpeed;
}

int writeport(int fd, char *chars) {
	int len = strlen(chars);
	chars[len] = 0x0d; // stick a <CR> after the command
	chars[len+1] = 0x00; // terminate the string properly
	int n = write(fd, chars, strlen(chars));
	if (n < 0) {
		fputs("write failed!\n", stderr);
		return 0;
	}
	return 1;
}

int readport(int fd, char *result) {
	result[0] = '\0';
	int iIn = read(fd, result, 254);
	if (iIn < 0) {
		if (errno == EAGAIN) {
			printf("SERIAL EAGAIN ERROR\n");
			return 0;
		} else {
//			printf("SERIAL read error %d %s\n", errno, strerror(errno));
			return 0;
		}
	}
	else
	{
		result[iIn] = '\0';
	}
	return 1;
}

SonarExpert::SonarExpert(ModuleDispatch* dis,long period) : StringModuleMember(dis, period, "SonarExpert")
{
	printf("\nSonarExpert built");
	fflush(stdout);
	
	lenbuffold = 0;

	for (short count = 0; count < SENSORS_NUMBER; count ++)
	{
		distances[count] = MAXDISTANCE;
	}
}

SonarExpert::~SonarExpert()
{
	printf("\nSonarExpert destroyed");
	fflush(stdout);
}

void SonarExpert::distancesVaried(int direction, int newDistance)
{
	printf("\nDistanze varied");
	fflush(stdout);

	distances[direction] = newDistance;

	NewMessage(MSG_TO_BRIAN, DISTANCES_DATA);
	mCurrentMessage<< "<D>"<<"NORTH "<<distances[NORTH]<<" "<<0.9<<" </D>\n";
	mCurrentMessage<< "<D>"<<"NORTHEAST "<<distances[NORTHEAST]<<" "<<0.9<<" </D>\n";
	mCurrentMessage<< "<D>"<<"EAST "<<distances[EAST]<<" "<<0.9<<" </D>\n";
	mCurrentMessage<< "<D>"<<"SOUTHEAST "<<distances[SOUTHEAST]<<" "<<0.9<<" </D>\n";
	mCurrentMessage<< "<D>"<<"SOUTH "<<distances[SOUTH]<<" "<<0.9<<" <D>\n";
	mCurrentMessage<< "<D>"<<"SOUTHWEST "<<distances[SOUTHWEST]<<" "<<0.9<<" </D>\n";
	mCurrentMessage<< "<D>"<<"WEST "<<distances[WEST]<<" "<<0.9<<" </D>\n";
	mCurrentMessage<< "<D>"<<"NORTHWEST "<<distances[NORTHWEST]<<" "<<0.9<<" </D>\n";
	CloseMessage();
	SendAllStringMessages();
}

void SonarExpert::RunInit()
{
	printf("\nSonarExpert initialized");
	fflush(stdout);
	
	sp = open("/dev/ttyUSB0",  O_RDWR | O_NOCTTY);
	if (sp == -1) {
		perror("open_port: Unable to open /dev/ttyUSB0 - ");
		return;
	}
	initport(sp);

	char r[] = "R";
	writeport(sp, r);
}

void SonarExpert::RunDuty()
{
	/*for (int count = 0, x; count < SENSORS_NUMBER; count++)
	{
		x = C1*(distances[count] - freshDistances[count])/freshDistances[count];
		if ( (x >= C2) || (x<=C2) )
		{
			distances[count] = freshDistances[count];
		}
	}*/
	
//	printf("\nSonarExpert: RunDuty. Dist: ");

	if (readport(sp, buff) != 1)
	{
		//printf("\nRicevuto: %s\n", buff);
		//for (int i = 0; buff[i] != '\0'; printf("%d ", buff[i]), i++);
		//printf("\nANALIZZATO:\n");
		return;
	}
	int c = 0;
	for (; buff[c] != '\0'; c++)
	{
		if (buff[c] == '+')
		{
			for (int numVirgole = 0, numLettura = 0, a = 0; buff[c] != '\n' && buff[c] != '\0'; c++)
			{
				if (numVirgole == 2)
				{
					int i = 0;
					for (; buff[c + i] != ',' && buff[c + i] != '\0' && buff[c + i] != '\n'; i++);
					if (buff[c + i] != '\0')
					{
						int k = 0;
						for (numLettura = 0; k < i; k++)
						{
							numLettura += (buff[c + k] - '0') * pow(10, (i - k - 1));
						}
					}
					c += i;
				}
				else if (numVirgole > 2)
				{
					int i = 0;
					for (; buff[c + i] != ',' && buff[c + i] != '\0' && buff[c + i] != '\n'; i++);
					if (buff[c + i] != '\0')
					{
						int k = 0;
						for (distances[(numLettura - 1) * 4 + numVirgole - 3] = 0; k < i; k++)
						{
							if (buff[c + k] != '0')
							{
								distances[(numLettura - 1) * 4 + numVirgole - 3] += (buff[c + k] - '0') * pow(10, (i - k - 1));
							}
						}
					}
					c += i;
				}
				
				if (buff[c] == ',')
				{	
					numVirgole ++;
				}
				if ((buff[c] == '\n') || (buff[c] == '\0'))
				{
					c--;
				}
			}
			if (buff[c] == '\0')
			{
				c--;
			}
		}
	}
	int i = c - 1;
	for (; buff[i] != '\n' && i > 0; i--);
	lenbuffold = c - i - 1;
	int cont = 0;
	for (i++; i < c; i++, cont++)
	{
		buff[cont] = buff[i];
	}
	
	for (c = 0; c < SENSORS_NUMBER; c++)
	{
		if (distances[c] == 0)
		{
			//distances[c] = MAXDISTANCE;
		}
		printf("\t%d ", distances[c]);
	}
	printf("\n");

	/*NewMessage(MSG_TO_BRIAN, DISTANCES_DATA);
	mCurrentMessage<< "<D>"<<"North "<<distances[NORTH]<<" "<<0.9<<" </D>\n";
	mCurrentMessage<< "<D>"<<"NorthEast "<<distances[NORTHEAST]<<" "<<0.9<<" </D>\n";
	mCurrentMessage<< "<D>"<<"East "<<distances[EAST]<<" "<<0.9<<" </D>\n";
	mCurrentMessage<< "<D>"<<"SouthEast "<<distances[SOUTHEAST]<<" "<<0.9<<" </D>\n";
	mCurrentMessage<< "<D>"<<"South "<<distances[SOUTH]<<" "<<0.9<<" </D>\n";
	mCurrentMessage<< "<D>"<<"SouthWest "<<distances[SOUTHWEST]<<" "<<0.9<<" </D>\n";
	mCurrentMessage<< "<D>"<<"West "<<distances[WEST]<<" "<<0.9<<" </D>\n";
	mCurrentMessage<< "<D>"<<"NorthWest "<<distances[NORTHWEST]<<" "<<0.9<<" </D>\n";
	CloseMessage();
	SendAllStringMessages();*/
}

int SonarExpert::getDistance(int num)
{
	return distances[num];
}

void SonarExpert::RunClose()
{
	printf("\nSonarExpert closed");
	fflush(stdout);
}
